@@ -28,8 +28,10 @@ def on_exit_B(event):
 
 
 class TestTransitions(TestCase):
+
     def setUp(self):
         self.stuff = Stuff()
+        self.machine_cls = Machine
 
     def tearDown(self):
         pass
@@ -50,7 +52,7 @@ class TestTransitions(TestCase):
              }
         ]
         s = Stuff()
-        m = Machine(model=s, states=states, transitions=transitions, initial='State2')
+        m = s.machine_cls(model=s, states=states, transitions=transitions, initial='State2')
         s.advance()
         self.assertEqual(s.message, 'Hello World!')
 
@@ -68,11 +70,11 @@ class TestTransitions(TestCase):
             {'trigger': 'run', 'source': 'B', 'dest': 'C'},
             {'trigger': 'sprint', 'source': 'C', 'dest': 'D'}
         ]
-        m = Machine(states=states, transitions=transitions, initial='A')
+        m = self.stuff.machine_cls(states=states, transitions=transitions, initial='A')
         self.assertEqual(m.initial, 'A')
-        m = Machine(states=states, transitions=transitions, initial='C')
+        m = self.stuff.machine_cls(states=states, transitions=transitions, initial='C')
         self.assertEqual(m.initial, 'C')
-        m = Machine(states=states, transitions=transitions)
+        m = self.stuff.machine_cls(states=states, transitions=transitions)
         self.assertEqual(m.initial, 'initial')
 
     def test_transition_definitions(self):
@@ -97,6 +99,14 @@ class TestTransitions(TestCase):
         m.sprint()
         self.assertEqual(m.state, 'D')
 
+    def test_add_states(self):
+        s = self.stuff
+        s.machine.add_state('X')
+        s.machine.add_state('Y')
+        s.machine.add_state('Z')
+        event = s.machine.events['to_{0}'.format(s.state)]
+        self.assertEqual(1, len(event.transitions['X']))
+
     def test_transitioning(self):
         s = self.stuff
         s.machine.add_transition('advance', 'A', 'B')
@@ -139,6 +149,14 @@ class TestTransitions(TestCase):
         s.advance()
         self.assertEqual(s.state, 'C')
 
+    def test_uncallable_callbacks(self):
+        s = self.stuff
+        s.machine.add_transition('advance', 'A', 'B', conditions=['property_that_fails', 'is_false'])
+        # make sure parameters passed by trigger events can be handled
+        s.machine.add_transition('advance', 'A', 'C', before=['property_that_fails', 'is_false'])
+        s.advance(level='MaximumSpeed')
+        self.assertTrue(s.is_C())
+
     def test_conditions_with_partial(self):
         def check(result):
             return result
@@ -373,6 +391,17 @@ class TestTransitions(TestCase):
         m.next_state()
         self.assertEqual(m.state, 'beginning')
 
+        # Partial state machine without the initial state
+        m = Machine('self', states, initial='beginning')
+        m.add_ordered_transitions(['middle', 'end'])
+        self.assertEqual(m.state, 'beginning')
+        with self.assertRaises(MachineError):
+            m.next_state()
+        m.to_middle()
+        for s in ('end', 'middle', 'end'):
+            m.next_state()
+            self.assertEqual(m.state, s)
+
     def test_ordered_transition_error(self):
         m = Machine(states=['A'], initial='A')
         with self.assertRaises(ValueError):
@@ -591,7 +620,7 @@ class TestTransitions(TestCase):
             m.do(machine=m)
 
     def test___getattr___and_identify_callback(self):
-        m = Machine(Stuff(), states=['A', 'B', 'C'], initial='A')
+        m = self.machine_cls(Stuff(), states=['A', 'B', 'C'], initial='A')
         m.add_transition('move', 'A', 'B')
         m.add_transition('move', 'B', 'C')
 
@@ -781,6 +810,13 @@ class TestTransitions(TestCase):
         with self.assertRaises(KeyError):
             self.stuff.trigger('do_raises_keyerror')
 
+        self.stuff.machine.get_model_state(self.stuff).ignore_invalid_triggers = True
+        self.stuff.trigger('should_not_raise_anything')
+        self.stuff.trigger('to_A')
+        self.assertTrue(self.stuff.is_A())
+        self.stuff.machine.ignore_invalid_triggers = True
+        self.stuff.trigger('should_not_raise_anything')
+
     def test_get_triggers(self):
         states = ['A', 'B', 'C']
         transitions = [['a2b', 'A', 'B'],
@@ -1055,6 +1091,17 @@ class TestTransitions(TestCase):
         self.assertEqual(instance.state_b, 'A')
         self.assertEqual(instance.state_a, 'B')
 
+    def test_initial_not_registered(self):
+        m1 = self.machine_cls(states=['A', 'B'], initial=self.machine_cls.state_cls('C'))
+        self.assertTrue(m1.is_C())
+        self.assertTrue('C' in m1.states)
+
+    def test_trigger_name_cannot_be_equal_to_model_attribute(self):
+        m = self.machine_cls(states=['A', 'B'])
+
+        with self.assertRaises(ValueError):
+            m.add_transition(m.model_attribute, "A", "B")
+
 
 class TestWarnings(TestCase):
     def test_warning(self):
